@@ -16,13 +16,10 @@ class Serializer(object):
     See https://github.com/bruth/json-graph-spec for more information.
     """
     def __init__(self):
+        self.stack = []
         self.indexes = {}
-        self.data = {
-            'nodes': [],
-            'rels': [],
-        }
-        self.node_index = 0
-        self.rel_index = 0
+        self.items = []
+        self.index = 0
 
     def _add_node(self, node):
         data = {'props': node.serialize()}
@@ -36,9 +33,9 @@ class Serializer(object):
         if node.update_props is not None:
             data['update'] = node.update_props
 
-        self.data['nodes'].append(data)
-        self.indexes[node.id] = self.node_index
-        self.node_index += 1
+        self.items.append(data)
+        self.indexes[node.id] = self.index
+        self.index += 1
 
     def _add_rel(self, rel):
         data = {
@@ -54,9 +51,9 @@ class Serializer(object):
         if rel.update_props is not None:
             data['update'] = rel.update_props
 
-        self.data['rels'].append(data)
-        self.indexes[rel.id] = self.rel_index
-        self.rel_index += 1
+        self.items.append(data)
+        self.indexes[rel.id] = self.index
+        self.index += 1
 
     def _serialize_rel(self, rel):
         if rel.id not in self.indexes:
@@ -68,29 +65,35 @@ class Serializer(object):
 
             self._add_rel(rel)
 
-    def _serialize_node(self, node, traverse=True):
+    def _serialize_node(self, node, traverse):
         if node.id not in self.indexes:
             self._add_node(node)
 
         if traverse:
             for rel in node.rels():
-                self._serialize_node(rel.end, traverse=traverse)
-                self._serialize_rel(rel)
+                self.stack.append(rel.end)
+                self.stack.append(rel)
 
-    def serialize(self, node, traverse=True):
+    def _serialize(self, item, traverse):
+        if isinstance(item, Node):
+            self._serialize_node(item, traverse)
+        else:
+            self._serialize_rel(item)
+
+    def serialize(self, item, traverse=True):
         "Prepares a node or relationship for export."
-        if isinstance(node, Node):
-            self._serialize_node(node, traverse=traverse)
-        elif isinstance(node, Rel):
-            self._serialize_rel(node)
-        elif isinstance(node, (tuple, list)):
-            for item in node:
-                self.serialize(item, traverse=traverse)
+        if isinstance(item, (Node, Rel)):
+            self.stack.append(item)
+        elif isinstance(item, (tuple, list)):
+            self.stack.extend(item)
         else:
             raise TypeError('unable to prepare objects with type "{}"'
-                            .format(type(node)))
+                            .format(type(item)))
 
-        return self.data
+        while self.stack:
+            self._serialize(self.stack.pop(), traverse)
+
+        return self.items
 
 
 def serialize(*args, **kwargs):
